<?php

namespace ImageAltAi\Admin;

class AdminPage {
    private static $instance = null;
    
    private function __construct() {
        add_action('admin_menu', [$this, 'addMenuPages']);
        add_action('admin_enqueue_scripts', [$this, 'enqueueAssets']);
        add_action('admin_enqueue_scripts', [$this, 'enqueueJS']);
        add_action('admin_enqueue_scripts', [$this, 'enqueueScripts']);
        
        // Add filter for admin page titles
        add_filter('admin_title', [$this, 'customizeAdminTitle'], 10, 2);
        // Add filter to modify admin footer text
        add_filter('admin_footer_text', [$this, 'customizeFooterText']);
    }
    
    public static function getInstance() {
        if (null === self::$instance) {
            self::$instance = new self();
        }
        return self::$instance;
    }
    
    public function addMenuPages() {
        // Add main menu under Media
        add_submenu_page(
            'upload.php',
            __('ImageAlt AI', 'imagealt-ai'),
            __('ImageAlt AI', 'imagealt-ai'),
            'manage_options',
            'imagealt-ai',
            [$this, 'renderPage']
        );
    }
    
    public function enqueueAssets($hook) {
        // Check for all our plugin pages
        if (!str_contains($hook, 'imagealt-ai') && !str_contains($hook, 'page=imagealt-ai')) {
            return;
        }
        
        wp_enqueue_style(
            'imagealt-ai-admin',
            IMAGEALT_AI_PLUGIN_URL . 'assets/css/style.css',
            [],
            IMAGEALT_AI_VERSION
        );
    }

    public function enqueueJS($hook) {
        // Check for media pages and our plugin pages
        if (!str_contains($hook, 'upload.php') 
            && !str_contains($hook, 'post.php') 
            && !str_contains($hook, 'imagealt-ai')) {
            return;
        }

        wp_enqueue_media();

        wp_enqueue_script(
            'imagealt-ai-admin',
            IMAGEALT_AI_PLUGIN_URL . 'assets/js/main.js',
            array('wp-i18n', 'jquery', 'media-views'),
            IMAGEALT_AI_VERSION,
            true
        );

        wp_localize_script('imagealt-ai-admin', 'imageAltAi', [
            'ajaxUrl' => admin_url('admin-ajax.php'),
            'nonce' => wp_create_nonce('imagealt-ai-nonce'),
            'intervals' => []
        ]);
    }
    
    public function enqueueScripts($hook) {
        // Check for all our plugin pages
        if (!str_contains($hook, 'page=imagealt-ai')) {
            return;
        }

        wp_enqueue_style('imagealt-ai-admin', IMAGEALT_AI_PLUGIN_URL . 'assets/css/admin.css', [], IMAGEALT_AI_VERSION);
        wp_enqueue_script('imagealt-ai-admin', IMAGEALT_AI_PLUGIN_URL . 'assets/js/admin.js', ['jquery'], IMAGEALT_AI_VERSION, true);
        
        wp_localize_script('imagealt-ai-admin', 'imageAltAi', [
            'ajaxUrl' => admin_url('admin-ajax.php'),
            'nonce' => wp_create_nonce('imagealt-ai-nonce')
        ]);
    }
    
    /**
     * Get the tabs configuration
     */
    private function getTabs() {
        $nonce_url = wp_create_nonce('imagealt_ai_tab_nonce');
        return [
            'dashboard' => [
                'title' => __('Dashboard', 'imagealt-ai'),
                'url' => wp_nonce_url(admin_url('upload.php?page=imagealt-ai'), 'imagealt_ai_tab_nonce'),
                'callback' => [$this, 'renderPage']
            ],
            'generator' => [
                'title' => __('Bulk Generator', 'imagealt-ai'),
                'url' => wp_nonce_url(admin_url('upload.php?page=imagealt-ai&tab=generator'), 'imagealt_ai_tab_nonce'),
                'callback' => [$this, 'renderPage']
            ],
            'settings' => [
                'title' => __('Settings', 'imagealt-ai'),
                'url' => wp_nonce_url(admin_url('upload.php?page=imagealt-ai&tab=settings'), 'imagealt_ai_tab_nonce'),
                'callback' => [$this, 'renderPage']
            ]
        ];
    }

    /**
     * Render the tabs navigation
     */
    private function renderTabs($current_page = 'dashboard') {
        $tabs = $this->getTabs();
        ?>
        <div class="nav-tab-wrapper">
            <?php foreach ($tabs as $tab_id => $tab) : 
                $active = ($current_page === $tab_id) ? 'nav-tab-active' : '';
                $tab_url = $tab['url'];
            ?>
                <a href="<?php echo esc_url($tab_url); ?>" 
                   class="nav-tab <?php echo esc_attr($active); ?>"
                   style="margin-left: 0; margin-right: 0.5em;">
                    <?php echo esc_html($tab['title']); ?>
                </a>
            <?php endforeach; ?>
        </div>
        <?php
    }
    
    public function renderPage() {
        $stats = \ImageAltAi\Core\Database::getInstance()->getStats();
        // phpcs:ignore WordPress.Security.NonceVerification.Recommended -- Nonce verification not required for read-only admin page navigation
        $tab = isset($_GET['tab']) ? sanitize_text_field(wp_unslash($_GET['tab'])) : 'dashboard';
        $this->renderTabs($tab);
        switch ($tab) {
            case 'dashboard':
                include IMAGEALT_AI_PLUGIN_DIR . 'templates/dashboard.php';
                break;
            case 'generator':
                include IMAGEALT_AI_PLUGIN_DIR . 'templates/bulk-generator.php';
                break;
            case 'settings':
                include IMAGEALT_AI_PLUGIN_DIR . 'templates/settings.php';
                break;
        }
    }
    /**
     * Customize the admin page title for our plugin pages
     */
    public function customizeAdminTitle($admin_title, $title) {
        $screen = get_current_screen();
        
        if (!$screen) {
            return $admin_title;
        }
        
        // Get the base page name from the screen ID
        // phpcs:ignore WordPress.Security.NonceVerification.Recommended -- Nonce verification not required for read-only admin page navigation
        $page = isset($_GET['page']) ? sanitize_text_field(wp_unslash($_GET['page'])) : '';
        // phpcs:ignore WordPress.Security.NonceVerification.Recommended -- Nonce verification not required for read-only admin page navigation
        $tab = isset($_GET['tab']) ? sanitize_text_field(wp_unslash($_GET['tab'])) : 'dashboard';

        if ($tab === 'settings') {
            return __('Settings', 'imagealt-ai') . ' - ' . get_bloginfo('name');
        }
        
        if ($tab === 'bulk-generator') {
            return __('Bulk Generator', 'imagealt-ai') . ' - ' . get_bloginfo('name');
        }
        
        if ($tab === 'dashboard') {
            return __('Image Alt AI', 'imagealt-ai') . ' - ' . get_bloginfo('name');
        }
        
        return $admin_title;
    }

    /**
     * Customize the admin footer text
     */
    public function customizeFooterText($text) {
        // Remove WordPress version globally
        remove_filter('update_footer', 'core_update_footer');
        
        // Check if we're on one of our plugin pages
        $screen = get_current_screen();
        // phpcs:ignore WordPress.Security.NonceVerification.Recommended -- Nonce verification not required for read-only admin page navigation
        $page = isset($_GET['page']) ? sanitize_text_field(wp_unslash($_GET['page'])) : '';
        
        // Verify it's our plugin page
        if ($screen && (strpos($screen->id, 'imagealt-ai') !== false || strpos($page, 'imagealt-ai') !== false)) {
            return '<div style="text-align: center;">Thank you for using <strong>ImageAlt AI Generator</strong> | Powered by <a href="https://converswp.com" target="_blank" style="color: rgb(79, 70, 229); text-decoration: none;">ConversWP</a></div>';
        }
        
        return '';
    }
}