<?php

namespace ImageAltAi\Core;

class Database {
    private static $instance = null;
    private $table_name;
    
    private function __construct() {
        global $wpdb;
        $this->table_name = $wpdb->prefix . 'imagealt_ai_logs';
    }
    
    public static function getInstance() {
        if (null === self::$instance) {
            self::$instance = new self();
        }
        return self::$instance;
    }
    
    public function createTables() {
        global $wpdb;
        
        $charset_collate = $wpdb->get_charset_collate();
        
        $sql = "CREATE TABLE IF NOT EXISTS {$this->table_name} (
            id bigint(20) NOT NULL AUTO_INCREMENT,
            attachment_id bigint(20) NOT NULL,
            original_alt text,
            generated_alt text NOT NULL,
            status varchar(20) NOT NULL,
            created_at datetime DEFAULT CURRENT_TIMESTAMP,
            updated_at datetime DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
            PRIMARY KEY  (id),
            KEY attachment_id (attachment_id)
        ) $charset_collate;";
        
        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        dbDelta($sql);
    }
    
    public function getStats() {
        global $wpdb;
        
        // Try to get cached stats first
        $stats = wp_cache_get('imagealt_ai_stats', 'imagealt-ai');
        if (false !== $stats) {
            return $stats;
        }
        
        // Using get_posts() for total images count
        $total_images = count(get_posts([
            'post_type' => 'attachment',
            'post_mime_type' => 'image',
            'posts_per_page' => -1,
            'fields' => 'ids',
        ]));
        
        // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery -- Custom query needed for performance with LEFT JOIN
        $images_without_alt = $wpdb->get_var("
            SELECT COUNT(*)
            FROM {$wpdb->posts} p
            LEFT JOIN {$wpdb->postmeta} pm ON p.ID = pm.post_id AND pm.meta_key = '_wp_attachment_image_alt'
            WHERE p.post_type = 'attachment'
            AND p.post_mime_type LIKE 'image/%'
            AND (pm.meta_value IS NULL OR pm.meta_value = '')
        ");
        
        // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery -- Custom table query necessary for plugin functionality
        $processed_images = $wpdb->get_var(
            $wpdb->prepare(
                "SELECT COUNT(DISTINCT attachment_id)
                FROM `" . esc_sql($this->table_name) . "`
                WHERE status = %s",
                'success'
            )
        );
        
        $stats = [
            'total_images' => (int)$total_images,
            'images_without_alt' => (int)$images_without_alt,
            'processed_images' => (int)$processed_images
        ];
        
        // Cache the results for 1 hour
        wp_cache_set('imagealt_ai_stats', $stats, 'imagealt-ai', HOUR_IN_SECONDS);
        
        return $stats;
    }
    
    public function logImageAltGeneration($attachment_id, $original_alt, $generated_alt, $status) {
        global $wpdb;
        
        // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery -- Custom table query necessary for plugin functionality
        $result = $wpdb->insert(
            $this->table_name,
            [
                'attachment_id' => $attachment_id,
                'original_alt' => $original_alt,
                'generated_alt' => $generated_alt,
                'status' => $status
            ],
            ['%d', '%s', '%s', '%s']
        );
        
        // Delete the stats cache when new data is added
        wp_cache_delete('imagealt_ai_stats', 'imagealt-ai');
        
        return $result;
    }
}