<?php

namespace ImageAltAi\Core;

class ImageProcessor {
    private static $instance = null;
    
    private function __construct() {
        $this->initHooks();
    }
    
    public static function getInstance() {
        if (null === self::$instance) {
            self::$instance = new self();
        }
        return self::$instance;
    }
    
    private function initHooks() {
        // Hook into media upload
        add_filter('wp_generate_attachment_metadata', [$this, 'processUploadedImage'], 10, 2);
        
        // Hook into block editor and content display
        add_filter('render_block', [$this, 'processBlockImage'], 10, 2);
        add_filter('the_content', [$this, 'processContentImages'], 999);
        
        // Add hooks for image rendering
        add_filter('wp_get_attachment_image_attributes', [$this, 'updateImageAttributes'], 20, 3);
        add_filter('image_send_to_editor', [$this, 'updateImageInEditor'], 20, 8);

        // Add hooks specifically for block editor
        add_filter('rest_prepare_attachment', [$this, 'prepareAttachmentForRest'], 10, 3);
        add_action('rest_after_insert_attachment', [$this, 'handleAttachmentSave'], 10, 2);
        add_filter('rest_pre_echo_response', [$this, 'modifyRestResponse'], 10, 3);
        
        // Register meta field for block editor
        register_post_meta('attachment', '_wp_attachment_image_alt', [
            'show_in_rest' => true,
            'single' => true,
            'type' => 'string',
            'auth_callback' => function() {
                return current_user_can('edit_posts');
            }
        ]);
        
        // Log process on view status if debugging is enabled
        add_action('wp_loaded', function() {
            if (defined('WP_DEBUG') && WP_DEBUG === true) {
                $status = get_option('imagealt_ai_process_on_view') ? 'enabled' : 'disabled';
                // Use wp_debug_log() if available (WP 5.1+)
                if (function_exists('wp_debug_log')) {
                    wp_debug_log("ImageAlt AI: Process on view option status: $status");
                } else if (defined('WP_DEBUG_LOG') && WP_DEBUG_LOG === true) {
                    // Fallback for older WordPress versions
                    error_log("ImageAlt AI: Process on view option status: $status"); // phpcs:ignore WordPress.PHP.DevelopmentFunctions.error_log_error_log
                }
            }
        });
    }
    
    public function processUploadedImage($metadata, $attachment_id) {
        if (!get_option('imagealt_ai_process_on_upload')) {
            return $metadata;
        }
        
        $this->generateImageAlt($attachment_id);
        
        return $metadata;
    }
    
    public function processBlockImage($block_content, $block) {
        if (!get_option('imagealt_ai_process_on_view')) {
            return $block_content;
        }

        // Only process core/image blocks
        if ($block['blockName'] !== 'core/image') {
            return $block_content;
        }

        // Get attachment ID from block attributes
        $attachment_id = isset($block['attrs']['id']) ? $block['attrs']['id'] : null;

        // If no ID in attributes, try to find it in the HTML
        if (!$attachment_id) {
            preg_match('/<img[^>]+wp-image-(\d+)[^>]+>/', $block_content, $matches);
            if (!empty($matches[1])) {
                $attachment_id = $matches[1];
            }
        }

        // check if already alt is there then skip the process
        $alt = get_post_meta($attachment_id, '_wp_attachment_image_alt', true);
        if (!empty($alt)) {
            return $block_content;
        }

        if ($attachment_id) {
            $current_alt = get_post_meta($attachment_id, '_wp_attachment_image_alt', true);
            
            // Generate alt text if empty
            if (empty($current_alt)) {
                $this->generateImageAlt($attachment_id);
            }

            $new_alt = get_post_meta($attachment_id, '_wp_attachment_image_alt', true);
            
            if (!empty($new_alt)) {
                // Update the block content
                $block_content = preg_replace(
                    '/(<img[^>]+)alt="[^"]*"([^>]*>)/',
                    '$1alt="' . esc_attr($new_alt) . '"$2',
                    $block_content
                );
                
                // If no alt attribute exists, add it
                if (strpos($block_content, 'alt="') === false) {
                    $block_content = preg_replace(
                        '/(<img[^>]+)(\/?>)/',
                        '$1 alt="' . esc_attr($new_alt) . '"$2',
                        $block_content
                    );
                }
            }
        }

        return $block_content;
    }
    
    public function processContentImages($content) {
        if (!get_option('imagealt_ai_process_on_view')) {
            return $content;
        }

        // Process images in content
        preg_match_all('/<img[^>]+>/', $content, $matches, PREG_OFFSET_CAPTURE);
        
        if (empty($matches[0])) {
            return $content;
        }

        // Process matches in reverse order to maintain correct offsets
        $matches[0] = array_reverse($matches[0]);
        
        foreach ($matches[0] as $match) {
            $img = $match[0];
            $offset = $match[1];
            $attachment_id = null;
            
            // Try to find wp-image-{id} class
            if (preg_match('/wp-image-(\d+)/', $img, $class_matches)) {
                $attachment_id = $class_matches[1];
            }
            
            // If no class found, try to find ID from src
            if (!$attachment_id && preg_match('/wp-content\/uploads\/[^"\']+/', $img, $src_matches)) {
                $src = $src_matches[0];
                $attachment = $this->get_attachment_id_from_src($src);
                if ($attachment) {
                    $attachment_id = $attachment;
                }
            }
            
            if ($attachment_id) {
                $current_alt = get_post_meta($attachment_id, '_wp_attachment_image_alt', true);
                
                // Generate alt text if empty
                if (empty($current_alt)) {
                    $this->generateImageAlt($attachment_id);
                    $new_alt = get_post_meta($attachment_id, '_wp_attachment_image_alt', true);
                    
                    if (!empty($new_alt)) {
                        // Update the alt attribute in the img tag
                        if (preg_match('/alt=(["\'])(.*?)\1/', $img)) {
                            $new_img = preg_replace(
                                '/alt=(["\'])(.*?)\1/',
                                'alt="' . esc_attr($new_alt) . '"',
                                $img
                            );
                        } else {
                            $new_img = str_replace('<img', '<img alt="' . esc_attr($new_alt) . '"', $img);
                        }
                        
                        // Replace the old img tag with the new one
                        $content = substr_replace($content, $new_img, $offset, strlen($img));
                    }
                }
            }
        }

        return $content;
    }
    
    private function get_attachment_id_from_src($src) {
        // First try WordPress's built-in function
        $attachment_id = attachment_url_to_postid($src);
        
        // If that fails, try our cached database method
        if (!$attachment_id) {
            // Create a unique cache key
            $cache_key = 'imagealt_ai_attachment_' . md5($src);
            
            // Try to get from cache first
            $attachment_id = wp_cache_get($cache_key, 'imagealt_ai');
            
            if (false === $attachment_id) {
                global $wpdb;
                
                // Clean up the src URL for better matching
                $src = preg_replace('/(-\d+x\d+)(\.[^.]+)$/', '$2', $src); // Remove image size from filename
                $src = str_replace('-scaled', '', $src); // Remove -scaled suffix
                
                // Try to find by filename first (more precise)
                $attachment_id = $wpdb->get_var( // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery, WordPress.DB.DirectDatabaseQuery.NoCaching
                    $wpdb->prepare(
                        "SELECT post_id FROM {$wpdb->postmeta} 
                        WHERE meta_key = '_wp_attached_file' 
                        AND meta_value LIKE %s",
                        '%' . $wpdb->esc_like(basename($src)) . '%'
                    )
                );
                
                // Cache the result for 12 hours
                wp_cache_set($cache_key, $attachment_id, 'imagealt_ai', 12 * HOUR_IN_SECONDS);
            }
        }
        
        return $attachment_id;
    }
    
    public function generateImageAlt($attachment_id) {
        try {
            // Skip if alt text exists
            $current_alt = get_post_meta($attachment_id, '_wp_attachment_image_alt', true);
            if (!empty($current_alt)) {
                return;
            }
            
            // Get image URL
            $image_url = wp_get_attachment_url($attachment_id);
            if (!$image_url) {
                throw new \Exception('Could not get image URL for attachment ID: ' . $attachment_id);
            }

            // Prepare prompt for pre- and post-text
            $prompt = Plugin::getInstance()->preparePrompt($attachment_id);
            
            $api = new \ImageAltAi\Services\OpenAIService();
            $alt_text = $api->generateImageAlt($image_url, $prompt);
            
            if ($alt_text) {
                // Update alt text
                update_post_meta($attachment_id, '_wp_attachment_image_alt', $alt_text);
                
                // Log success
                Database::getInstance()->logImageAltGeneration(
                    $attachment_id,
                    $current_alt,
                    $alt_text,
                    'success'
                );
            } else {
                throw new \Exception('API returned empty alt text for attachment ID: ' . $attachment_id);
            }
        } catch (\Exception $e) {
            Database::getInstance()->logImageAltGeneration(
                $attachment_id,
                $current_alt ?? '',
                $e->getMessage(),
                'error'
            );
        }
    }

    public function updateImageAttributes($attr, $attachment, $size) {
        if (!get_option('imagealt_ai_process_on_view')) {
            return $attr;
        }

        // Check if alt text is empty
        if (empty($attr['alt'])) {
            $this->generateImageAlt($attachment->ID);
            $new_alt = get_post_meta($attachment->ID, '_wp_attachment_image_alt', true);
            if (!empty($new_alt)) {
                $attr['alt'] = $new_alt;
                
                // Also update the block attributes if this is a block editor request
                if (defined('REST_REQUEST') && REST_REQUEST) {
                    $post = get_post($attachment->ID);
                    if ($post) {
                        $blocks = parse_blocks($post->post_content);
                        foreach ($blocks as &$block) {
                            if ($block['blockName'] === 'core/image' && isset($block['attrs']['id']) && $block['attrs']['id'] == $attachment->ID) {
                                $block['attrs']['alt'] = $new_alt;
                            }
                        }
                        wp_update_post([
                            'ID' => $post->ID,
                            'post_content' => serialize_blocks($blocks)
                        ]);
                    }
                }
            }
        }

        return $attr;
    }

    public function updateImageInEditor($html, $id, $caption, $title, $align, $url, $size, $alt) {
        if (!get_option('imagealt_ai_process_on_view')) {
            return $html;
        }

        if (empty($alt)) {
            $this->generateImageAlt($id);
            $new_alt = get_post_meta($id, '_wp_attachment_image_alt', true);
            if (!empty($new_alt)) {
                $html = preg_replace(
                    '/alt=(["\'])(.*?)\1/',
                    'alt="' . esc_attr($new_alt) . '"',
                    $html
                );
            }
        }

        return $html;
    }

    public function handleAttachmentSave($attachment, $request) {
        if (!get_option('imagealt_ai_process_on_view')) {
            return;
        }

        $current_alt = get_post_meta($attachment->ID, '_wp_attachment_image_alt', true);
        if (empty($current_alt)) {
            $this->generateImageAlt($attachment->ID);
        }
    }

    public function modifyRestResponse($response, $rest_server, $request) {
        if (!get_option('imagealt_ai_process_on_view')) {
            return $response;
        }

        // Only modify responses for block editor requests
        if (strpos($request->get_route(), '/wp/v2/media') === false) {
            return $response;
        }

        // Handle both single and multiple media items
        if (isset($response['id'])) {
            $this->updateMediaResponse($response);
        } elseif (is_array($response)) {
            foreach ($response as &$item) {
                if (isset($item['id'])) {
                    $this->updateMediaResponse($item);
                }
            }
        }

        return $response;
    }

    private function updateMediaResponse(&$item) {
        $attachment_id = $item['id'];
        $current_alt = get_post_meta($attachment_id, '_wp_attachment_image_alt', true);

        if (empty($current_alt)) {
            $this->generateImageAlt($attachment_id);
            $new_alt = get_post_meta($attachment_id, '_wp_attachment_image_alt', true);
            if (!empty($new_alt)) {
                $item['alt_text'] = $new_alt;
                $item['caption']['raw'] = $item['caption']['raw'] ?: $new_alt;
                $item['caption']['rendered'] = $item['caption']['rendered'] ?: $new_alt;
                
                // Update block attributes
                if (isset($item['attributes'])) {
                    $item['attributes']['alt'] = $new_alt;
                }
            }
        } else {
            $item['alt_text'] = $current_alt;
            if (isset($item['attributes'])) {
                $item['attributes']['alt'] = $current_alt;
            }
        }
    }

    public function prepareAttachmentForRest($response, $post, $request) {
        if (!get_option('imagealt_ai_process_on_view')) {
            return $response;
        }

        $data = $response->get_data();
        $current_alt = get_post_meta($post->ID, '_wp_attachment_image_alt', true);

        if (empty($current_alt)) {
            $this->generateImageAlt($post->ID);
            $new_alt = get_post_meta($post->ID, '_wp_attachment_image_alt', true);
            if (!empty($new_alt)) {
                $data['alt_text'] = $new_alt;
                $data['alt'] = $new_alt; // Add alt directly to data
                
                // Also update media details
                if (isset($data['media_details'])) {
                    $data['media_details']['alt_text'] = $new_alt;
                }
                
                $response->set_data($data);
            }
        } else {
            $data['alt_text'] = $current_alt;
            $data['alt'] = $current_alt; // Add alt directly to data
            if (isset($data['media_details'])) {
                $data['media_details']['alt_text'] = $current_alt;
            }
            $response->set_data($data);
        }

        return $response;
    }
}