<?php

namespace ImageAltAi\Core;

use WP_Query;

class Plugin {
    private static $instance = null;
    
    private function __construct() {
        $this->init();
    }
    
    public static function getInstance() {
        if (null === self::$instance) {
            self::$instance = new self();
        }
        return self::$instance;
    }
    
    private function init() {
        // Initialize components
        \ImageAltAi\Admin\AdminPage::getInstance();

        Database::getInstance();
        ImageProcessor::getInstance();
        
        // Register activation hook
        register_activation_hook(IMAGEALT_AI_PLUGIN_DIR . 'imagealt-ai.php', [$this, 'activate']);
        
        // Register deactivation hook
        register_deactivation_hook(IMAGEALT_AI_PLUGIN_DIR . 'imagealt-ai.php', [$this, 'deactivate']);
        
        // Add settings
        add_action('admin_init', [$this, 'registerSettings']);
        
        // Add AJAX handlers
        add_action('wp_ajax_generate_alt_text', [$this, 'handleGenerateImageAlt']);
        add_action('wp_ajax_save_alt_text', [$this, 'handleSaveImageAlt']);
        // Add AJAX handler for fetching images without alt text
        add_action('wp_ajax_fetch_images_without_alt', [$this, 'fetchImagesWithoutAlt']);

        // Add media button
//        add_filter('attachment_fields_to_edit', [$this, 'addAiButton'], 10, 2);


    }
    
    public function registerSettings() {
        register_setting('imagealt_ai_settings', 'imagealt_ai_openai_api_key', 'sanitize_text_field');
        register_setting('imagealt_ai_settings', 'imagealt_ai_vision_ai_key', 'sanitize_text_field');
        register_setting('imagealt_ai_settings', 'imagealt_ai_selected_ai', [$this, 'sanitize_selected_ai']); // phpcs:disable PluginCheck.CodeAnalysis.SettingSanitization.register_settingDynamic
        register_setting('imagealt_ai_settings', 'imagealt_ai_pre_text', 'sanitize_textarea_field');
        register_setting('imagealt_ai_settings', 'imagealt_ai_post_text', 'sanitize_textarea_field');
        register_setting('imagealt_ai_settings', 'imagealt_ai_process_on_upload', '__return_bool');
        register_setting('imagealt_ai_settings', 'imagealt_ai_process_on_view', '__return_bool');
    }

    /**
     * Sanitize checkbox values to boolean.
     *
     * @param $input
     * @return bool
     */
    public static function sanitize_checkbox( $input ): bool {
        return filter_var( $input, FILTER_VALIDATE_BOOLEAN );
    }

    /**
     * Sanitize and validate selected AI option.
     *
     * @param string $input
     * @return string
     */
    public static function sanitize_selected_ai(string $input ): string {
        $allowed_values = ['openai', 'visionai'];
        return in_array( $input, $allowed_values, true ) ? $input : 'openai';
    }

	/**
	 * @throws \Exception
	 */
	public function handleGenerateImageAlt() {
        check_ajax_referer('imagealt-ai-nonce', 'nonce');
        
        $attachment_id = isset($_POST['attachment_id']) ? absint($_POST['attachment_id']) : wp_send_json_error(['message' => 'Missing attachment ID']);

        $auto_save = isset($_POST['auto_save']) ? filter_var(wp_unslash($_POST['auto_save'], FILTER_VALIDATE_BOOLEAN)) : false;
        
        if (!$attachment_id) {
            wp_send_json_error(['message' => 'Invalid attachment ID']);
        }
        
        $selected_ai = get_option('imagealt_ai_selected_ai', 'openai');
        $prompt = $this->preparePrompt($attachment_id);
        
        $service = $selected_ai === 'openai' 
            ? new \ImageAltAi\Services\OpenAIService()
            : new \ImageAltAi\Services\VisionAiService();
        
        $image_url = wp_get_attachment_url($attachment_id);
        
        $alt_text = $service->generateImageAlt($image_url, $prompt);
        
        if ($alt_text) {
            
            if ($auto_save) {
                update_post_meta($attachment_id, '_wp_attachment_image_alt', $alt_text);
            }

            $original_alt = get_post_meta($attachment_id, '_wp_attachment_image_alt', true);
            Database::getInstance()->logImageAltGeneration($attachment_id, $original_alt, $alt_text, 'success');
            
            wp_send_json_success(['alt_text' => $alt_text]);
        } else {
            wp_send_json_error(['message' => 'Failed to generate alt text']);
        }

        return;
    }
    
    public function handleSaveImageAlt() {
        try {
            check_ajax_referer('imagealt-ai-nonce', 'nonce');
            
            $attachment_id = isset($_POST['attachment_id']) ? absint($_POST['attachment_id']) : wp_send_json_error(['message' => 'Missing attachment ID']);
            
            $alt_text = isset($_POST['alt_text']) ? sanitize_text_field(wp_unslash($_POST['alt_text'])) : '';
            
            if (!$attachment_id) {
                wp_send_json_error(['message' => 'Invalid attachment ID']);
                return;
            }

            if (empty($alt_text)) {
                wp_send_json_error(['message' => 'Alt text cannot be empty']);
                return;
            }

            // Verify attachment exists
            if (!get_post($attachment_id)) {
                wp_send_json_error(['message' => 'Attachment not found']);
                return;
            }
            
            // Get original alt text before update
            $original_alt = get_post_meta($attachment_id, '_wp_attachment_image_alt', true);
            
            // Update the meta value
            $result = update_post_meta($attachment_id, '_wp_attachment_image_alt', $alt_text);

            wp_send_json_success([
                'message' => 'Alt text saved successfully',
                'alt_text' => $alt_text
            ]);
            
            return;

            // update_post_meta returns:
            // - false if the new value is the same as the old value
            // - true if the value was updated
            // - meta ID if the value was added
            // So we need to check if the value was actually saved correctly
            // $updated_alt = get_post_meta($attachment_id, '_wp_attachment_image_alt', true);
            
            // if ($updated_alt === $alt_text) {
            //     // Log successful save
            //     Database::getInstance()->logImageAltGeneration(
            //         $attachment_id, 
            //         $original_alt, 
            //         $alt_text, 
            //         'success'
            //     );
                
            //     wp_send_json_success([
            //         'message' => 'Alt text saved successfully',
            //         'alt_text' => $alt_text
            //     ]);
            //     return;
            // }
            
            // wp_send_json_error(['message' => 'Failed to verify alt text update']);
            
        } catch (\Exception $e) {
            // error_log('ImageAlt AI Save Error: ' . $e->getMessage());
            wp_send_json_error([
                'message' => 'Error saving alt text: ' . $e->getMessage()
            ]);
        }
    }

    public function addAiButton($form_fields, $post) {

		if (strpos($post->post_mime_type, 'image') === false) {
			return $form_fields;
		}

		$form_fields['alt']['input'] = 'html';
		$form_fields['alt']['label'] = __('Generate with AI', 'imagealt-ai');
		$form_fields['alt']['html'] = sprintf(
			'<span id="imagealt-ai-attachment-button" class="setting" data-setting="alt">' .
			'<button onload="javascript::moveGenerateWithAiToTop();" type="button" class="button imagealt-ai-generate" data-id="%d" style="line-height: 1.5;float: right;">%s</button></span>',
			$post->ID,
			__('⚡ Generate with AI', 'imagealt-ai')
		);
		return $form_fields;
	}

    public function activate() {
        // Create database tables
        Database::getInstance()->createTables();

        // Set default options
        $this->setDefaultOptions();

        // Redirect after activation
        add_option('imagealt_ai_do_activation_redirect', true);
    }

    public function deactivate() {
        // Cleanup if needed
    }

    private function setDefaultOptions() {
        $defaults = [
            'openai_api_key' => '',
            'vision_ai_key' => '',
            'auto_generate' => false,
            'pre_text' => '',
            'post_text' => '',
            'process_on_upload' => true,
            'process_on_view' => false
        ];

        foreach ($defaults as $key => $value) {
            if (false === get_option("imagealt_ai_{$key}")) {
                add_option("imagealt_ai_{$key}", $value);
            }
        }
    }

    public function preparePrompt($attachment_id): string
    {
		$site_name = get_bloginfo('name');
		$site_description = get_bloginfo('description');
		$image_title = get_the_title($attachment_id);

        $pre_text = get_option('imagealt_ai_pre_text', '');
        $post_text = get_option('imagealt_ai_post_text', '');

        $prefix = ' For website name: "' . $site_name . '" and description: "' . $site_description . '".';
        $prefix .= ' Start alt text with ';

        $suffix = 'Dont add . at the end but add ';

		$prompt = 'Do not return error message or guide in the alt text. return directly the alt text.';
        // check if pre- and post-text are not empty and prepare prompt accordingly
        if (!empty($pre_text) && !empty($post_text)) {
            $prompt .= $prefix . $pre_text . ' and ' . $suffix . $post_text;
        } elseif (!empty($pre_text)) {
            $prompt .= $prefix . $pre_text;
        } elseif (!empty($post_text)) {
            $prompt .= $suffix . $post_text;
        }
        $prompt .= '. If you cant make a decision, If possible use image title: ' . $image_title;

        return $prompt;
    }

    public function fetchImagesWithoutAlt(): void
    {
        check_ajax_referer('imagealt-ai-nonce', 'nonce');

        // Validate and sanitize input with defaults
        $page = isset($_POST['page']) ? absint($_POST['page']) : 1;
        $per_page = isset($_POST['per_page']) ? absint($_POST['per_page']) : 10;
        
        // Ensure reasonable limits
        $per_page = min(max($per_page, 1), 100); // Between 1 and 100
        $page = max($page, 1); // At least 1
        
        $offset = ($page - 1) * $per_page;

        global $wpdb;

        // Try to get cached results first
        $cache_key = 'imagealt_ai_images_without_alt_' . $page . '_' . $per_page;
        $results = wp_cache_get($cache_key, 'imagealt-ai');

        if (false === $results) {
            // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery -- Complex query optimization for performance
            $results = $wpdb->get_results($wpdb->prepare("
                SELECT p.ID, p.guid 
                FROM {$wpdb->posts} p
                LEFT JOIN {$wpdb->postmeta} pm ON p.ID = pm.post_id 
                    AND pm.meta_key = '_wp_attachment_image_alt'
                WHERE p.post_type = 'attachment'
                AND p.post_mime_type LIKE %s
                AND (pm.meta_value IS NULL OR pm.meta_value = '')
                GROUP BY p.ID
                LIMIT %d OFFSET %d
            ", 'image/%', $per_page, $offset));

            // Cache the results for 1 hour
            wp_cache_set($cache_key, $results, 'imagealt-ai', HOUR_IN_SECONDS);
        }

        // Get total count from cache or database
        $total_cache_key = 'imagealt_ai_images_without_alt_total';
        $total = wp_cache_get($total_cache_key, 'imagealt-ai');

        if (false === $total) {
            // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery -- Complex query optimization for performance
            $total = $wpdb->get_var($wpdb->prepare("
                SELECT COUNT(DISTINCT p.ID)
                FROM {$wpdb->posts} p
                LEFT JOIN {$wpdb->postmeta} pm ON p.ID = pm.post_id 
                    AND pm.meta_key = '_wp_attachment_image_alt'
                WHERE p.post_type = 'attachment'
                AND p.post_mime_type LIKE %s
                AND (pm.meta_value IS NULL OR pm.meta_value = '')
            ", 'image/%'));

            wp_cache_set($total_cache_key, $total, 'imagealt-ai', HOUR_IN_SECONDS);
        }

        $images = array_map(function($result) {
            return [
                'id' => $result->ID,
                'url' => $result->guid
            ];
        }, $results);

        wp_send_json_success([
            'images' => $images,
            'total_remaining' => (int)$total
        ]);
    }
}