<?php
if (!defined('ABSPATH')) exit;

// Verify nonce if items_per_page is set
$items_per_page = 10; // Default value
if (isset($_GET['items_per_page'])) {
    $nonce = isset($_GET['_wpnonce']) ? sanitize_text_field(wp_unslash($_GET['_wpnonce'])) : '';
    if (wp_verify_nonce($nonce, 'imagealt_ai_items_per_page')) {
        $items_per_page = intval($_GET['items_per_page']);
    }
}

// Get the current page from the query string, default to 1 if not set
$current_page = isset($_GET['paged']) ? max(1, intval($_GET['paged'])) : 1;

// Query attachments with pagination
$images = new WP_Query([
    'post_type' => 'attachment',
    'post_mime_type' => 'image',
    'posts_per_page' => $items_per_page,
    'post_status' => 'inherit',
    'paged' => $current_page,
]);

// Calculate total pages
$total_pages = $images->max_num_pages;
?>

<!-- Toast Container -->
<div class="toast-container"></div>

<script>
jQuery(document).ready(function ($) {
    // Items per page change handler
    $('#items-per-page').change(function () {
        var itemsPerPage = $(this).val();
        var url = new URL(window.location.href);
        url.searchParams.set('items_per_page', itemsPerPage);
        url.searchParams.set('paged', 1); // Reset to the first page
        url.searchParams.set('_wpnonce', '<?php echo esc_js(wp_create_nonce('imagealt_ai_items_per_page')); ?>');
        window.location.href = url.toString();
    });

    // Bulk select functionality
    $('#bulk-select-all').on('change', function () {
        const isChecked = $(this).prop('checked');
        $('input[name="bulk-select[]"]').prop('checked', isChecked);
        updateBulkButtons();
        if (isChecked) {
            window.ImageAltToast.success('All images selected');
        }
    });

    $('input[name="bulk-select[]"]').on('change', function () {
        updateBulkButtons();
    });

    function updateBulkButtons() {
        const hasSelection = $('input[name="bulk-select[]"]:checked').length > 0;
        $('#generate-selected, #save-all').prop('disabled', !hasSelection);
    }

    // Single image generation - Modified to prevent double calls
    $('.generate-single-manual').off('click').on('click', function (e) {
        e.preventDefault();
        e.stopPropagation();
        
        const button = $(this);
        if (button.data('processing')) {
            return false;
        }
        
        const id = button.data('id');
        generateImageAlt(id);
    });

    // Single image save
    $('.save-single').on('click', function () {
        const button = $(this);
        const id = button.data('id');
        const imageAlt = $(`textarea[name="alt_text[${id}]"]`).val();
        saveImageAlt(id, imageAlt);
    });

    // Bulk generation
    $('#generate-selected').on('click', function () {
        const selectedIds = $('input[name="bulk-select[]"]:checked').map(function () {
            return $(this).val();
        }).get();

        if (selectedIds.length === 0) {
            window.ImageAltToast.error('Please select images first');
            return;
        }

        window.ImageAltToast.success('Starting bulk generation...');
        const progressBar = $('.progress-bar');
        progressBar.show();

        let processed = 0;
        const total = selectedIds.length;

        function processNext() {
            if (processed >= total) {
                progressBar.hide();
                window.ImageAltToast.success('Bulk generation completed');
                return;
            }

            const id = selectedIds[processed];
            generateImageAlt(id, function () {
                processed++;
                updateProgress(processed, total);
                processNext();
            });
        }

        processNext();
    });

    // Save all changes
    $('#save-all').on('click', function () {
        const selectedRows = $('input[name="bulk-select[]"]:checked').closest('tr');
        if (selectedRows.length === 0) {
            window.ImageAltToast.error('Please select images first');
            return;
        }

        window.ImageAltToast.success('Starting bulk save...');
        let savedCount = 0;
        let hasErrors = false;

        selectedRows.each(function () {
            const id = $(this).data('id');
            const imageAlt = $(`textarea[name="alt_text[${id}]"]`).val();
            
            if (!imageAlt || imageAlt.trim() === '') {
                window.ImageAltToast.error(`Empty alt text for image ${id}`);
                hasErrors = true;
                return;
            }

            saveImageAlt(id, imageAlt, function() {
                savedCount++;
                if (savedCount === selectedRows.length && !hasErrors) {
                    window.ImageAltToast.success('All changes saved successfully');
                }
            });
        });
    });

    // Helper functions - Modified generateImageAlt
    function generateImageAlt(id, callback) {
        const row = $(`tr[data-id="${id}"]`);
        const status = row.find('.status');
        const input = row.find('.generated-alt-text');
        const saveButton = row.find('.save-single');
        const generateButton = row.find('.generate-single');

        // Prevent duplicate calls
        if (generateButton.data('processing') || status.html().includes('pending')) {
            return;
        }

        // Set processing flag
        generateButton.data('processing', true);
        status.html('<span class="pending">Generating...</span>');

        // Clear any existing content
        input.val('');
        saveButton.prop('disabled', true);

        $.ajax({
            url: imageAltAi.ajaxUrl,
            type: 'POST',
            data: {
                action: 'generate_alt_text',
                nonce: imageAltAi.nonce,
                attachment_id: id,
                auto_save: false,
                context: 'bulk'
            },
            success: function (response) {
                if (response.success) {
                    input.val(response.data.alt_text);
                    status.html('<span class="success">Generated</span>');
                    saveButton.prop('disabled', false);
                    window.ImageAltToast.success('Alt text generated successfully');
                } else {
                    status.html('<span class="error">Error: ' + response.data.message + '</span>');
                    window.ImageAltToast.error(response.data.message || 'Failed to generate alt text');
                }
            },
            error: function () {
                status.html('<span class="error">Failed to generate</span>');
                window.ImageAltToast.error('Failed to generate alt text');
            },
            complete: function () {
                // Reset processing flag
                generateButton.data('processing', false);
                if (typeof callback === 'function') {
                    callback();
                }
            }
        });
    }

    function saveImageAlt(id, imageAlt, callback) {
        if (!id || !imageAlt || imageAlt.trim() === '') {
            window.ImageAltToast.error('Alt text cannot be empty');
            return;
        }

        const row = $(`tr[data-id="${id}"]`);
        const status = row.find('.status');
        const saveButton = row.find('.save-single');
        const currentAltField = row.find('.current-alt-text');

        status.html('<span class="pending">Saving...</span>');
        saveButton.prop('disabled', true);

        $.ajax({
            url: imageAltAi.ajaxUrl,
            type: 'POST',
            data: {
                action: 'save_alt_text',
                nonce: imageAltAi.nonce,
                attachment_id: id,
                alt_text: imageAlt.trim()
            },
            success: function (response) {
                if (response.success && response.data && response.data.alt_text) {
                    status.html('<span class="success">Saved</span>');
                    currentAltField.val(response.data.alt_text);
                    saveButton.prop('disabled', true);
                    window.ImageAltToast.success('Alt text saved successfully');
                } else {
                    status.html('<span class="error">Error: ' + (response.data?.message || 'Failed to save alt text') + '</span>');
                    saveButton.prop('disabled', false);
                    window.ImageAltToast.error(response.data?.message || 'Failed to save alt text');
                }
            },
            error: function (xhr, status, error) {
                console.error('Save error:', error);
                status.html('<span class="error">Failed to save</span>');
                saveButton.prop('disabled', false);
                window.ImageAltToast.error('Failed to save alt text: ' + error);
            },
            complete: function () {
                if (typeof callback === 'function') {
                    callback();
                }
            }
        });
    }

    function updateProgress(current, total) {
        const percentage = Math.round((current / total) * 100);
        $('.progress').css('width', percentage + '%');
        $('.progress-text').text(percentage + '%');
    }
});
</script>

<div class="imagealt-ai-action-wrap imagealt-ai-bulk-generator bg-background rounded-lg border border-solid border-[rgb(228,228,231)]  ">
    <div class="imagealt-ai-action-card p-8 ">

        <div class="flex flex-row justify-between">
            <div>
                <h2 class="text-2xl mt-0 mb-0 font-bold tracking-tight"><?php esc_html_e('Bulk Alt Text Generator', 'imagealt-ai'); ?></h2>
                <p class="max-w-2xl text-base pb-6 font-light text-gray-500"><?php esc_html_e('Generate alt text for multiple images at once. You can generate alt text for all images or select specific images to generate alt text for.', 'imagealt-ai'); ?></p>
            </div>
            <div class="plugin-logo">
                <?php
                    printf(
                        '<img src="%s" alt="%s" class="%s" style="%s" loading="lazy">', // phpcs:ignore PluginCheck.CodeAnalysis.ImageFunctions.NonEnqueuedImage
                        esc_url(IMAGEALT_AI_PLUGIN_URL . 'assets/images/logo.png'),
                        esc_attr__('ImageAlt AI Logo', 'imagealt-ai'),
                        esc_attr('w-24 h-auto'),
                        esc_attr('max-width: 150px; height: auto;')
                    );
                ?>
            </div>
        </div>

        <div class="imagealt-ai-bulk-controls flex items-center justify-between space-x-4">
            <div class="flex-1 space-x-2">
                <button 
                    class="inline-flex items-center justify-center whitespace-nowrap rounded-md text-sm font-medium transition-colors focus-visible:outline-none focus-visible:ring-1 focus-visible:ring-ring disabled:pointer-events-none disabled:opacity-50 bg-primary text-primary-foreground shadow hover:bg-primary/90 h-9 px-4 py-2"
                    style="background: rgb(79, 70, 229); color: white;"
                    id="generate-selected" 
                    disabled
                >
                    <?php esc_html_e('Generate for Selected', 'imagealt-ai'); ?>
                </button>
                <button 
                    class="inline-flex items-center justify-center whitespace-nowrap rounded-md text-sm font-medium transition-colors focus-visible:outline-none focus-visible:ring-1 focus-visible:ring-ring disabled:pointer-events-none disabled:opacity-50 bg-primary text-primary-foreground shadow hover:bg-primary/90 h-9 px-4 py-2"
                    style="background: rgb(79, 70, 229); color: white;"
                    id="save-all" 
                    disabled
                >
                    <?php esc_html_e('Save All Changes', 'imagealt-ai'); ?>
                </button>
            </div>

            <div class="flex items-center space-x-2">
                <div class="relative w-[200px]">
                    <svg xmlns="http://www.w3.org/2000/svg" 
                        class="absolute left-3 top-1/2 -translate-y-1/2 h-4 w-4 text-gray-400" 
                        fill="none" 
                        viewBox="0 0 24 24" 
                        stroke="currentColor"
                    >
                        <path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M21 21l-6-6m2-5a7 7 0 11-14 0 7 7 0 0114 0z" />
                    </svg>
                    <input 
                        type="search" 
                        id="image-search" 
                        class="flex h-9 w-full rounded-md border border-input bg-background pl-9 pr-3 py-2 text-sm ring-offset-background file:border-0 file:bg-transparent file:text-sm file:font-medium placeholder:text-muted-foreground placeholder:pl-0 focus-visible:outline-none focus-visible:ring-1 focus-visible:ring-ring disabled:cursor-not-allowed disabled:opacity-50"
                        placeholder="<?php esc_attr_e('  Search...', 'imagealt-ai'); ?>"
                        style="text-indent: 20px;"
                    >
                </div>
            </div>

            <div class="space-x-2 flex items-center">
                <select id="items-per-page"
                        class="inline-flex h-9 w-[100px] items-center justify-between rounded-md border border-input bg-transparent px-3 py-2 text-sm shadow-sm ring-offset-background placeholder:text-muted-foreground focus:outline-none focus:ring-1 focus:ring-ring disabled:cursor-not-allowed disabled:opacity-50"
                >
                    <?php
                    $items_per_page_options = [2, 5, 10, 20, 50];
                    foreach ($items_per_page_options as $option) {
                        $selected = ($option == $items_per_page) ? 'selected' : '';
                        // translators: %d: Number of items to display per page
                        echo '<option value="' . esc_attr($option) . '" ' . esc_attr($selected) . '>' . sprintf(esc_html__('Show:&nbsp;&nbsp;%d', 'imagealt-ai'), esc_html($option)) . '</option>';
                    }
                    ?>
                </select>
            </div>
        </div>

        <div class="progress-bar" style="display: none;">
            <div class="progress"></div>
            <span class="progress-text">0%</span>
        </div>

        <div class="rounded-md border">
            <div class="relative w-full overflow-auto">
                <table class="w-full caption-bottom text-sm wp-list-table striped">
                    <thead class="[&_tr]:border-b">
                        <tr class="border-b transition-colors hover:bg-muted/50 data-[state=selected]:bg-muted">
                            <th class="check-column h-10 px-2 text-left align-middle font-medium text-muted-foreground [&:has([role=checkbox])]:pr-0 [&>[role=checkbox]]:translate-y-[2px] [&:has([role=checkbox])]:pl-3">
                                <input type="checkbox" id="bulk-select-all">
                            </th>
                            <th class="h-10 px-2 text-left align-middle font-medium text-muted-foreground [&:has([role=checkbox])]:pr-0 [&>[role=checkbox]]:translate-y-[2px] [&:has([role=checkbox])]:pl-3"><?php esc_html_e('Image', 'imagealt-ai'); ?></th>
                            <th class="h-10 px-2 text-left align-middle font-medium text-muted-foreground [&:has([role=checkbox])]:pr-0 [&>[role=checkbox]]:translate-y-[2px] [&:has([role=checkbox])]:pl-3"><?php esc_html_e('Current Alt Text', 'imagealt-ai'); ?></th>
                            <th class="h-10 px-2 text-left align-middle font-medium text-muted-foreground [&:has([role=checkbox])]:pr-0 [&>[role=checkbox]]:translate-y-[2px] [&:has([role=checkbox])]:pl-3"><?php esc_html_e('Generated Alt Text', 'imagealt-ai'); ?></th>
                            <th class="h-10 px-2 text-left align-middle font-medium text-muted-foreground [&:has([role=checkbox])]:pr-0 [&>[role=checkbox]]:translate-y-[2px] [&:has([role=checkbox])]:pl-3"
                                style="width: 150px;"><?php esc_html_e('Actions', 'imagealt-ai'); ?></th>
                            <th class="h-10 px-2 text-left align-middle font-medium text-muted-foreground [&:has([role=checkbox])]:pr-0 [&>[role=checkbox]]:translate-y-[2px] [&:has([role=checkbox])]:pl-3"
                                style="width: 150px;"><?php esc_html_e('Status', 'imagealt-ai'); ?></th>
                        </tr>
                    </thead>

                    <tbody class="[&_tr:last-child]:border-0">
                        <?php if ($images->have_posts()): ?>
                            <?php while ($images->have_posts()): $images->the_post(); ?>
                                <?php
                                $current_alt = get_post_meta(get_the_ID(), '_wp_attachment_image_alt', true);
                                $thumbnail = wp_get_attachment_image(get_the_ID(), 'thumbnail');
                                ?>

                                <tr data-id="<?php echo esc_attr(get_the_ID()); ?>"
                                    class="border-b transition-colors hover:bg-muted/50 data-[state=selected]:bg-muted">
                                    <td class="p-2 align-middle [&:has([role=checkbox])]:pr-0 [&>[role=checkbox]]:translate-y-[2px] [&:has([role=checkbox])]:pl-3">
                                        <input type="checkbox" name="bulk-select[]"
                                            value="<?php echo esc_attr(get_the_ID()); ?>">
                                    </td>
                                    <td class="p-2 align-middle [&:has([role=checkbox])]:pr-0 [&>[role=checkbox]]:translate-y-[2px] [&:has([role=checkbox])]:pl-3">
                                        <?php echo wp_kses_post($thumbnail); ?>
                                    </td>
                                    <td class="p-2 align-middle [&:has([role=checkbox])]:pr-0 [&>[role=checkbox]]:translate-y-[2px] [&:has([role=checkbox])]:pl-3">
                                        <textarea
                                            class="current-alt-text flex min-h-[60px] w-full rounded-md border border-input bg-transparent px-3 py-2 text-base shadow-sm placeholder:text-muted-foreground focus-visible:outline-none focus-visible:ring-1 focus-visible:ring-ring disabled:cursor-not-allowed disabled:opacity-50 md:text-sm"
                                            readonly
                                            rows="3"><?php echo esc_attr($current_alt); ?>
                                        </textarea>
                                    </td>
                                    <td class="p-2 align-middle [&:has([role=checkbox])]:pr-0 [&>[role=checkbox]]:translate-y-[2px] [&:has([role=checkbox])]:pl-3">
                                        <textarea
                                            class="generated-alt-text flex min-h-[60px] w-full rounded-md border border-input bg-transparent px-3 py-2 text-base shadow-sm placeholder:text-muted-foreground focus-visible:outline-none focus-visible:ring-1 focus-visible:ring-ring disabled:cursor-not-allowed disabled:opacity-50 md:text-sm"
                                            name="alt_text[<?php echo esc_attr(get_the_ID()); ?>]"
                                            placeholder="<?php esc_attr_e('Click generate to create alt text', 'imagealt-ai'); ?>"
                                            rows="3">
                                        </textarea>
                                    </td>
                                    <td class="p-2 align-middle [&:has([role=checkbox])]:pr-0 [&>[role=checkbox]]:translate-y-[2px] [&:has([role=checkbox])]:pl-3">
                                        <button class="button generate-single-manual"
                                                data-id="<?php echo esc_attr(get_the_ID()); ?>">
                                            <?php esc_html_e('Generate', 'imagealt-ai'); ?>
                                        </button>
                                        <button class="button save-single" data-id="<?php echo esc_attr(get_the_ID()); ?>">
                                            <?php esc_html_e('Save', 'imagealt-ai'); ?>
                                        </button>
                                    </td>
                                    
                                    <td class="p-2 align-middle [&:has([role=checkbox])]:pr-0 [&>[role=checkbox]]:translate-y-[2px] [&:has([role=checkbox])]:pl-3">
                                        <span class="status"></span>
                                    </td>
                                </tr>
                            <?php endwhile; ?>
                        <?php else: ?>
                            <tr>
                                <td colspan="5"><?php esc_html_e('No images found.', 'imagealt-ai'); ?></td>
                            </tr>
                        <?php endif; ?>
                    </tbody>
                </table>
            </div>
        </div>

        <div class="tablenav">
            <div class="tablenav-pages flex justify-center items-center mt-4" role="navigation" aria-label="pagination">
                <?php
                    echo wp_kses_post(paginate_links(
                        array(
                            'base' => add_query_arg(
                                array(
                                    'paged' => '%#%',
                                )
                            ),
                            'format' => '',
                            'prev_text' => '<span class="inline-flex items-center justify-center rounded-md text-sm transition-colors border-input hover:bg-accent hover:text-accent-foreground h-6 w-6" aria-label="' . esc_attr__('Previous page', 'imagealt-ai') . '">' . esc_html__('‹', 'imagealt-ai') . '</span>',
                            'next_text' => '<span class="inline-flex items-center justify-center rounded-md text-sm transition-colors border-input hover:bg-accent hover:text-accent-foreground h-6 w-6" aria-label="' . esc_attr__('Next page', 'imagealt-ai') . '">' . esc_html__('›', 'imagealt-ai') . '</span>',
                            'before_page_number' => '',
                            'after_page_number' => '',
                            'total' => $total_pages,
                            'current' => $current_page,
                            'type' => 'plain',
                            'mid_size' => 1,
                            'end_size' => 1,
                            'prev_next' => true,
                            'add_args' => false,
                            'add_fragment' => '',
                        )
                    ));
                ?>
            </div>
        </div>
    </div>
</div>